#pragma once

#include <stdint.h>
#include <avr/io.h>


namespace Devices::Mcu
{
    struct Timer0
    {
        enum class Mode
        {
            Mode0,
            Mode1,
            Mode2,
            Mode3,
            Mode5,
            Mode7,
            
            Normal = Mode0,
            PhaseCorrectPwm1 = Mode1,
            ClearOnCompareMatch = Mode2,
            FastPwm3 = Mode3,
            PhaseCorrectPwm5 = Mode5,
            FastPwm7 = Mode7
        };


        enum class ClockSource
        {
            Disabled = 0x00,
            ClkDiv1,
            ClkDiv8,
            ClkDiv64,
            ClkDiv256,
            ClkDiv1024,
            ClkExternalFalling,
            ClkExternalRising
        };


        static void SetMode(Mode const mode)
        {
            switch (mode)
            {
                case Mode::Normal:
                    TCCR0B &= ~_BV(WGM02);
                    TCCR0A &= ~_BV(WGM01) & ~_BV(WGM00);
                    break;
                    
                case Mode::PhaseCorrectPwm1:
                    TCCR0B &= ~_BV(WGM02);
                    TCCR0A &= ~_BV(WGM01);
                    TCCR0A |= _BV(WGM00);
                    break;
                    
                case Mode::ClearOnCompareMatch:
                    TCCR0B &= ~_BV(WGM02);
                    TCCR0A |= _BV(WGM01);
                    TCCR0A &= ~_BV(WGM00);
                    break;
                    
                case Mode::FastPwm3:
                    TCCR0B &= ~_BV(WGM02);
                    TCCR0A |= _BV(WGM01) | _BV(WGM00);;
                    break;

                case Mode::PhaseCorrectPwm5:
                    TCCR0B |= _BV(WGM02);
                    TCCR0A &= ~_BV(WGM01);
                    TCCR0A |= _BV(WGM00);
                    break;

                case Mode::FastPwm7:
                    TCCR0B |= _BV(WGM02);
                    TCCR0A |= _BV(WGM01) | _BV(WGM00);
                    break;                    
            }
        }


        static void SetClockSource(ClockSource const clockSource)
        {
            TCCR0B &= ~_BV(CS02) & ~_BV(CS01) & ~_BV(CS00);
            TCCR0B |= (uint8_t) clockSource;
        }


        static void SetInterrupts(bool overflow, bool outputCompareA, bool outputCompareB)
        {
            if (overflow)
            {
                TIMSK0 |= _BV(TOIE0);
            }
            else
            {
                TIMSK0 &= ~_BV(TOIE0);
            }

            if (outputCompareA)
            {
                TIMSK0 |= _BV(OCIE0A);
            }
            else
            {
                TIMSK0 &= ~_BV(OCIE0A);
            }

            if (outputCompareB)
            {
                TIMSK0 |= _BV(OCIE0B);
            }
            else
            {
                TIMSK0 &= ~_BV(OCIE0B);
            }
        }


        static void SetValue(uint8_t const value)
        {
            TCNT0 = value;
        }


        static void SetOutputCompareA(uint8_t const value)
        {
            OCR0A = value;
        }


        static bool IsOverflown()
        {
            return (TIFR0 & _BV(TOV2)) != 0;
        }
    };
}